@@ -54,7 +54,7 @@ impl EnumVariant {
         is_tagged: bool,
         variant: &syn::Variant,
         generic_params: GenericParams,
-        mod_cfg: &Option<Cfg>,
+        mod_cfg: Option<&Cfg>,
     ) -> Result<Self, String> {
         let discriminant = match variant.discriminant {
             Some((_, ref expr)) => match value_from_expr(expr) {
@@ -250,7 +250,7 @@ impl Enum {
         }
     }
 
-    pub fn load(item: &syn::ItemEnum, mod_cfg: &Option<Cfg>) -> Result<Enum, String> {
+    pub fn load(item: &syn::ItemEnum, mod_cfg: Option<&Cfg>) -> Result<Enum, String> {
         let repr = Repr::load(&item.attrs)?;
         if repr == Repr::RUST {
             return Err("Enum not marked with a valid repr(prim) or repr(C).".to_owned());
@@ -332,8 +332,8 @@ impl Item for Enum {
         &self.export_name
     }
 
-    fn cfg(&self) -> &Option<Cfg> {
-        &self.cfg
+    fn cfg(&self) -> Option<&Cfg> {
+        self.cfg.as_ref()
     }
 
     fn annotations(&self) -> &AnnotationSet {
@@ -511,8 +511,15 @@ impl Source for Enum {
         // If tagged, we need to emit a proper struct/union wrapper around our enum
         self.generic_params.write(config, out);
         if is_tagged && config.language == Language::Cxx {
-            out.write(if separate_tag { "struct " } else { "union " });
-            write!(out, "{}", self.export_name());
+            out.write(if separate_tag { "struct" } else { "union" });
+
+            if self.annotations.must_use {
+                if let Some(ref anno) = config.structure.must_use {
+                    write!(out, " {}", anno)
+                }
+            }
+
+            write!(out, " {}", self.export_name());
             out.open_brace();
         }
 
@@ -534,10 +541,17 @@ impl Source for Enum {
                 write!(out, " {}", enum_name);
             }
         } else {
+            out.write("enum class");
+
+            if self.annotations.must_use {
+                if let Some(ref anno) = config.enumeration.must_use {
+                    write!(out, " {}", anno)
+                }
+            }
+
+            write!(out, " {}", enum_name);
             if let Some(prim) = size {
-                write!(out, "enum class {} : {}", enum_name, prim);
-            } else {
-                write!(out, "enum class {}", enum_name);
+                write!(out, " : {}", prim);
             }
         }
         out.open_brace();
@@ -647,9 +661,8 @@ impl Source for Enum {
             let skip_fields = if separate_tag { 0 } else { 1 };
 
             // Emit convenience methods
-            if config.language == Language::Cxx
-                && config.enumeration.derive_helper_methods(&self.annotations)
-            {
+            let derive_helper_methods = config.enumeration.derive_helper_methods(&self.annotations);
+            if config.language == Language::Cxx && derive_helper_methods {
                 for variant in &self.variants {
                     out.new_line();
                     out.new_line();
@@ -687,28 +700,28 @@ impl Source for Enum {
                         for &(ref field_name, ref ty, ..) in body.fields.iter().skip(skip_fields) {
                             out.new_line();
                             match ty {
-                                Type::Array(ref _ty, ref length) => {
+                                Type::Array(ref ty, ref length) => {
                                     // arrays are not assignable in C++ so we
                                     // need to manually copy the elements
+                                    write!(out, "for (int i = 0; i < {}; i++)", length.as_str());
+                                    out.open_brace();
                                     write!(
                                         out,
-                                        "for (int i = 0; i < {}; i++) {{\
-                                         result.{}.{}[i] = {}[i];\
-                                         }}",
-                                        length.as_str(),
-                                        variant_name,
-                                        field_name,
-                                        arg_renamer(field_name)
+                                        "::new (&result.{}.{}[i]) (",
+                                        variant_name, field_name
                                     );
+                                    ty.write(config, out);
+                                    write!(out, ")({}[i]);", arg_renamer(field_name));
+                                    out.close_brace(false);
                                 }
-                                _ => {
+                                ref ty => {
                                     write!(
                                         out,
-                                        "result.{}.{} = {};",
-                                        variant_name,
-                                        field_name,
-                                        arg_renamer(field_name)
+                                        "::new (&result.{}.{}) (",
+                                        variant_name, field_name
                                     );
+                                    ty.write(config, out);
+                                    write!(out, ")({});", arg_renamer(field_name));
                                 }
                             }
                         }
@@ -733,16 +746,85 @@ impl Source for Enum {
                 }
             }
 
+            let derive_const_casts = config.enumeration.derive_const_casts(&self.annotations);
+            let derive_mut_casts = config.enumeration.derive_mut_casts(&self.annotations);
+            if config.language == Language::Cxx
+                && derive_helper_methods
+                && (derive_const_casts || derive_mut_casts)
+            {
+                let assert_name = match config.enumeration.cast_assert_name {
+                    Some(ref n) => &**n,
+                    None => "assert",
+                };
+
+                for variant in &self.variants {
+                    let (member_name, body) = match variant.body {
+                        Some((ref member_name, ref body)) => (member_name, body),
+                        None => continue,
+                    };
+
+                    let field_count = body.fields.len() - skip_fields;
+                    if field_count == 0 {
+                        continue;
+                    }
+
+                    let dig = field_count == 1 && body.tuple_struct;
+                    let mut derive_casts = |const_casts: bool| {
+                        out.new_line();
+                        out.new_line();
+
+                        if dig {
+                            let field = body.fields.iter().skip(skip_fields).next().unwrap();
+                            let return_type = field.1.clone();
+                            let return_type = if const_casts {
+                                Type::Ref(Box::new(return_type))
+                            } else {
+                                Type::MutRef(Box::new(return_type))
+                            };
+                            return_type.write(config, out);
+                        } else {
+                            if const_casts {
+                                write!(out, "const {}&", body.export_name());
+                            } else {
+                                write!(out, "{}&", body.export_name());
+                            }
+                        }
+
+                        write!(out, " As{}()", variant.export_name);
+                        if const_casts {
+                            write!(out, " const");
+                        }
+                        out.open_brace();
+                        write!(out, "{}(Is{}());", assert_name, variant.export_name);
+                        out.new_line();
+                        if dig {
+                            write!(out, "return {}._0;", member_name);
+                        } else {
+                            write!(out, "return {};", member_name);
+                        }
+                        out.close_brace(false);
+                    };
+
+                    if derive_const_casts {
+                        derive_casts(true)
+                    }
+
+                    if derive_mut_casts {
+                        derive_casts(false)
+                    }
+                }
+            }
+
+            let other = if let Some(r) = config.function.rename_args {
+                r.apply_to_snake_case("other", IdentifierType::FunctionArg)
+            } else {
+                String::from("other")
+            };
+
             if config.language == Language::Cxx
                 && self.can_derive_eq()
                 && config.structure.derive_eq(&self.annotations)
             {
-                let other = if let Some(r) = config.function.rename_args {
-                    r.apply_to_snake_case("other", IdentifierType::FunctionArg)
-                } else {
-                    String::from("other")
-                };
-
                 out.new_line();
                 out.new_line();
                 write!(
@@ -790,6 +872,72 @@ impl Source for Enum {
                 }
             }
 
+            if config.language == Language::Cxx
+                && config
+                    .enumeration
+                    .derive_tagged_enum_destructor(&self.annotations)
+            {
+                out.new_line();
+                out.new_line();
+                write!(out, "~{}()", self.export_name);
+                out.open_brace();
+                write!(out, "switch (tag)");
+                out.open_brace();
+                for variant in &self.variants {
+                    if let Some((ref variant_name, ref item)) = variant.body {
+                        write!(
+                            out,
+                            "case {}::{}: {}.~{}(); break;",
+                            self.tag.as_ref().unwrap(),
+                            variant.export_name,
+                            variant_name,
+                            item.export_name(),
+                        );
+                        out.new_line();
+                    }
+                }
+                write!(out, "default: break;");
+                out.close_brace(false);
+                out.close_brace(false);
+            }
+
+            if config.language == Language::Cxx
+                && config
+                    .enumeration
+                    .derive_tagged_enum_copy_constructor(&self.annotations)
+            {
+                out.new_line();
+                out.new_line();
+                write!(
+                    out,
+                    "{}(const {}& {})",
+                    self.export_name, self.export_name, other
+                );
+                out.new_line();
+                write!(out, " : tag({}.tag)", other);
+                out.open_brace();
+                write!(out, "switch (tag)");
+                out.open_brace();
+                for variant in &self.variants {
+                    if let Some((ref variant_name, ref item)) = variant.body {
+                        write!(
+                            out,
+                            "case {}::{}: ::new (&{}) ({})({}.{}); break;",
+                            self.tag.as_ref().unwrap(),
+                            variant.export_name,
+                            variant_name,
+                            item.export_name(),
+                            other,
+                            variant_name,
+                        );
+                        out.new_line();
+                    }
+                }
+                write!(out, "default: break;");
+                out.close_brace(false);
+                out.close_brace(false);
+            }
+
             if let Some(body) = config.export.extra_body(&self.path) {
                 out.write_raw_block(body);
             }
